@@ -61,12 +61,15 @@ namespace italian // language namespace
 		{"Clock skew", "Orologio disallineato"},
 		{"Offline", "Disconnesso"},
 		{"Symmetric NAT", "NAT simmetrico"},
+		{"Full cone NAT", "Cono completo NAT"},
+		{"No Descriptors", "Nessun descrittore"},
 		{"Uptime", "In funzione da"},
 		{"Network status", "Stato della rete"},
 		{"Network status v6", "Stato della rete v6"},
 		{"Stopping in", "Arresto in"},
 		{"Family", "Famiglia"},
 		{"Tunnel creation success rate", "Percentuale di tunnel creati con successo"},
+		{"Total tunnel creation success rate", "Percentuale di successo totale nella creazione del tunnel"},
 		{"Received", "Ricevuti"},
 		{"%.2f KiB/s", "%.2f KiB/s"},
 		{"Sent", "Inviati"},
@@ -93,6 +96,7 @@ namespace italian // language namespace
 		{"Address", "Indirizzo"},
 		{"Type", "Tipologia"},
 		{"EncType", "Tipo di crittografia"},
+		{"Expire LeaseSet", "Scadenza LeaseSet"},
 		{"Inbound tunnels", "Tunnel in entrata"},
 		{"%dms", "%dms"},
 		{"Outbound tunnels", "Tunnel in uscita"},
@@ -107,6 +111,7 @@ namespace italian // language namespace
 		{"Local Destination", "Destinazione locale"},
 		{"Streams", "Flussi"},
 		{"Close stream", "Interrompi il flusso"},
+		{"Such destination is not found", "Questa destinazione non è stata trovata"},
 		{"I2CP session not found", "Sessione I2CP non trovata"},
 		{"I2CP is not enabled", "I2CP non è abilitato"},
 		{"Invalid", "Invalido"},
@@ -116,6 +121,7 @@ namespace italian // language namespace
 		{"Gateway", "Gateway"},
 		{"TunnelID", "TunnelID"},
 		{"EndDate", "Data di fine"},
+		{"floodfill mode is disabled", "la modalità floodfill è disabilitata"},
 		{"Queue size", "Dimensione della coda"},
 		{"Run peer test", "Esegui il test dei peer"},
 		{"Reload tunnels configuration", "Ricarica la configurazione dei tunnel"},
@@ -147,6 +153,8 @@ namespace italian // language namespace
 		{"StreamID can't be null", "Lo StreamID non può essere null"},
 		{"Return to destination page", "Ritorna alla pagina di destinazione"},
 		{"You will be redirected in %d seconds", "Sarai reindirizzato tra %d secondi"},
+		{"LeaseSet expiration time updated", "Tempo di scadenza LeaseSet aggiornato"},
+		{"LeaseSet is not found or already expired", "LeaseSet non trovato o già scaduto"},
 		{"Transit tunnels count must not exceed %d", "Il conteggio dei tunnel di transito non deve superare %d"},
 		{"Back to commands list", "Ritorna alla lista dei comandi"},
 		{"Register at reg.i2p", "Registra a reg.i2p"},
@@ -155,7 +163,6 @@ namespace italian // language namespace
 		{"Submit", "Invia"},
 		{"Domain can't end with .b32.i2p", "I domini non possono terminare con .b32.i2p"},
 		{"Domain must end with .i2p", "I domini devono terminare con .i2p"},
-		{"Such destination is not found", "Questa destinazione non è stata trovata"},
 		{"Unknown command", "Comando sconosciuto"},
 		{"Command accepted", "Comando accettato"},
 		{"Proxy error", "Errore del proxy"},
@@ -165,8 +172,15 @@ namespace italian // language namespace
 		{"You may try to find this host on jump services below", "Si può provare a trovare questo host sui servizi di salto qui sotto"},
 		{"Invalid request", "Richiesta non valida"},
 		{"Proxy unable to parse your request", "Il proxy non è in grado di elaborare la tua richiesta"},
+		{"Addresshelper is not supported", "Addresshelper non è supportato"},
+		{"Host %s is <font color=red>already in router's addressbook</font>. <b>Be careful: source of this URL may be harmful!</b> Click here to update record: <a href=\"%s%s%s&update=true\">Continue</a>.", "L'host %s è <font color=red>già nella rubrica del router</font>. <b>Attenzione: la fonte di questo URL potrebbe essere dannosa!</b> Fai clic qui per aggiornare il record: <a href=\"%s%s%s&update=true\">Continua</a>."},
+		{"Addresshelper forced update rejected", "Aggiornamento forzato dell'helper degli indirizzi rifiutato"},
 		{"To add host <b>%s</b> in router's addressbook, click here: <a href=\"%s%s%s\">Continue</a>.", "Per aggiungere host <b>%s</b> nella rubrica del router, clicca qui: <a href=\"%s%s%s\">Continua</a>."},
+		{"Addresshelper request", "Richiesta di indirizzo helper"},
+		{"Host %s added to router's addressbook from helper. Click here to proceed: <a href=\"%s\">Continue</a>.", "L'host %s viene aggiunto alla rubrica del router dall'helper. Fai clic qui per procedere: <a href=\"%s\">Continua</a>."},
+		{"Addresshelper adding", "Aggiunta di Addresshelper"},
 		{"Host %s is <font color=red>already in router's addressbook</font>. Click here to update record: <a href=\"%s%s%s&update=true\">Continue</a>.", "L'host %s è <font color=red>già nella rubrica del router</font>. Clicca qui per aggiornare il record: <a href=\"%s%s%s&update=true\">Continua</a>."},
+		{"Addresshelper update", "Aggiornamento dell'helper degli indirizzi"},
 		{"Invalid request URI", "URI della richiesta non valido"},
 		{"Can't detect destination host from request", "Impossibile determinare l'host di destinazione dalla richiesta"},
 		{"Outproxy failure", "Fallimento del proxy di uscita"},
