@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2021-2022, The PurpleI2P Project
+* Copyright (c) 2021-2023, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -31,15 +31,16 @@ namespace turkmen // language namespace
 
 	static std::map<std::string, std::string> strings
 	{
-		{"KiB", "KiB"},
-		{"MiB", "MiB"},
-		{"GiB", "GiB"},
+		{"%.2f KiB", "%.2f KiB"},
+		{"%.2f MiB", "%.2f MiB"},
+		{"%.2f GiB", "%.2f GiB"},
 		{"building", "bina"},
 		{"failed", "şowsuz"},
 		{"expiring", "möhleti gutarýar"},
 		{"established", "işleýär"},
 		{"unknown", "näbelli"},
 		{"exploratory", "gözleg"},
+		{"Purple I2P Webconsole", "Web konsoly Purple I2P"},
 		{"<b>i2pd</b> webconsole", "Web konsoly <b>i2pd</b>"},
 		{"Main page", "Esasy sahypa"},
 		{"Router commands", "Marşrutizator buýruklary"},
@@ -57,7 +58,6 @@ namespace turkmen // language namespace
 		{"Unknown", "Näbelli"},
 		{"Proxy", "Proksi"},
 		{"Mesh", "MESH-tor"},
-		{"Error", "Ýalňyşlyk"},
 		{"Clock skew", "Takyk wagt däl"},
 		{"Offline", "Awtonom"},
 		{"Symmetric NAT", "Simmetriklik NAT"},
@@ -68,7 +68,7 @@ namespace turkmen // language namespace
 		{"Family", "Maşgala"},
 		{"Tunnel creation success rate", "Gurlan teneller üstünlikli gurlan teneller"},
 		{"Received", "Alnan"},
-		{"KiB/s", "KiB/s"},
+		{"%.2f KiB/s", "%.2f KiB/s"},
 		{"Sent", "Ýerleşdirildi"},
 		{"Transit", "Tranzit"},
 		{"Data path", "Maglumat ýoly"},
@@ -94,7 +94,7 @@ namespace turkmen // language namespace
 		{"Type", "Görnüş"},
 		{"EncType", "Şifrlemek görnüşi"},
 		{"Inbound tunnels", "Gelýän tuneller"},
-		{"ms", "ms"},
+		{"%dms", "%dms"},
 		{"Outbound tunnels", "Çykýan tuneller"},
 		{"Tags", "Bellikler"},
 		{"Incoming", "Gelýän"},
@@ -116,7 +116,6 @@ namespace turkmen // language namespace
 		{"Gateway", "Derweze"},
 		{"TunnelID", "Tuneliň ID"},
 		{"EndDate", "Gutarýar"},
-		{"not floodfill", "fludfil däl"},
 		{"Queue size", "Nobatyň ululygy"},
 		{"Run peer test", "Synag başlaň"},
 		{"Decline transit tunnels", "Tranzit tunellerini ret ediň"},
@@ -146,8 +145,6 @@ namespace turkmen // language namespace
 		{"Destination not found", "Niýetlenen ýeri tapylmady"},
 		{"StreamID can't be null", "StreamID boş bolup bilmez"},
 		{"Return to destination page", "Barmaly nokadynyň nokadyna gaýdyp geliň"},
-		{"You will be redirected in 5 seconds", "5 sekuntdan soň täzeden ugrukdyrylarsyňyz"},
-		{"Transit tunnels count must not exceed 65535", "Tranzit tagtalaryň sany 65535-den geçmeli däldir"},
 		{"Back to commands list", "Topar sanawyna dolan"},
 		{"Register at reg.i2p", "Reg.i2P-de hasaba duruň"},
 		{"Description", "Beýany"},
@@ -165,32 +162,24 @@ namespace turkmen // language namespace
 		{"You may try to find this host on jump services below", "Aşakdaky böküş hyzmatlarynda bu öý eýesini tapmaga synanyşyp bilersiňiz"},
 		{"Invalid request", "Nädogry haýyş"},
 		{"Proxy unable to parse your request", "Proksi haýyşyňyzy derňäp bilmeýär"},
-		{"addresshelper is not supported", "Salgylandyryjy goldanok"},
-		{"Host", "Adres"},
-		{"added to router's addressbook from helper", "marşruteriň adresini kömekçiden goşdy"},
-		{"Click here to proceed:", "Dowam etmek bu ýerde basyň:"},
-		{"Continue", "Dowam et"},
-		{"Addresshelper found", "Forgelper tapyldy"},
-		{"already in router's addressbook", "marşruteriň adres kitaby"},
-		{"Click here to update record:", "Recordazgyny täzelemek üçin bu ýerde basyň:"},
-		{"invalid request uri", "nädogry haýyş URI"},
+		{"Invalid request URI", "Nädogry haýyş URI"},
 		{"Can't detect destination host from request", "Haýyşdan barmaly ýerini tapyp bilemok"},
 		{"Outproxy failure", "Daşarky proksi ýalňyşlyk"},
-		{"bad outproxy settings", "daşarky daşarky proksi sazlamalary nädogry"},
-		{"not inside I2P network, but outproxy is not enabled", "I2P torunda däl, ýöne daşarky proksi goşulmaýar"},
-		{"unknown outproxy url", "näbelli daşarky proksi URL"},
-		{"cannot resolve upstream proxy", "has ýokary proksi kesgitläp bilmeýär"},
-		{"hostname too long", "hoster eýesi ady gaty uzyn"},
-		{"cannot connect to upstream socks proxy", "ýokary jorap SOCKS proksi bilen birigip bolmaýar"},
-		{"Cannot negotiate with socks proxy", "Iň ýokary jorap SOCKS proksi bilen ylalaşyp bilmeýärler"},
+		{"Bad outproxy settings", "Daşarky Daşarky proksi sazlamalary nädogry"},
+		{"Host %s is not inside I2P network, but outproxy is not enabled", "Adres %s I2P torunda däl, ýöne daşarky proksi goşulmaýar"},
+		{"Unknown outproxy URL", "Näbelli daşarky proksi URL"},
+		{"Cannot resolve upstream proxy", "Has ýokary proksi kesgitläp bilmeýär"},
+		{"Hostname is too long", "Hoster eýesi ady gaty uzyn"},
+		{"Cannot connect to upstream SOCKS proxy", "Ýokary jorap SOCKS proksi bilen birigip bolmaýar"},
+		{"Cannot negotiate with SOCKS proxy", "Iň ýokary jorap SOCKS proksi bilen ylalaşyp bilmeýärler"},
 		{"CONNECT error", "Bagyr haýyşy säwligi"},
-		{"Failed to Connect", "Birikdirip bilmedi"},
-		{"socks proxy error", "socks proksi ýalňyşlygy"},
-		{"failed to send request to upstream", "öý eýesi proksi üçin haýyş iberip bilmedi"},
-		{"No Reply From socks proxy", "Jorap proksi serwerinden hiç hili jogap ýok"},
-		{"cannot connect", "birikdirip bilmedi"},
-		{"http out proxy not implemented", "daşarky HTTP proksi serwerini goldamak amala aşyrylmaýar"},
-		{"cannot connect to upstream http proxy", "ýokary akym HTTP proksi serwerine birigip bilmedi"},
+		{"Failed to connect", "Birikdirip bilmedi"},
+		{"SOCKS proxy error", "SOCKS proksi ýalňyşlygy"},
+		{"Failed to send request to upstream", "Öý eýesi proksi üçin haýyş iberip bilmedi"},
+		{"No reply from SOCKS proxy", "Jorap SOCKS proksi serwerinden hiç hili jogap ýok"},
+		{"Cannot connect", "Birikdirip bilmedi"},
+		{"HTTP out proxy not implemented", "Daşarky HTTP proksi serwerini goldamak amala aşyrylmaýar"},
+		{"Cannot connect to upstream HTTP proxy", "Ýokary jorap HTTP proksi bilen birigip bolmaýar"},
 		{"Host is down", "Salgy elýeterli däl"},
 		{"Can't create connection to requested host, it may be down. Please try again later.", "Talap edilýän salgyda birikmäni gurup bilmedim, onlaýn bolup bilmez. Soňra haýyşy soň gaýtalamaga synanyşyň."},
 		{"", ""},
@@ -198,10 +187,10 @@ namespace turkmen // language namespace
 
 	static std::map<std::string, std::vector<std::string>> plurals
 	{
-		{"days",    {"gün", "gün"}},
-		{"hours",   {"sagat", "sagat"}},
-		{"minutes", {"minut", "minut"}},
-		{"seconds", {"sekunt", "sekunt"}},
+		{"%d days",    {"%d gün", "%d gün"}},
+		{"%d hours",   {"%d sagat", "%d sagat"}},
+		{"%d minutes", {"%d minut", "%d minut"}},
+		{"%d seconds", {"%d sekunt", "%d sekunt"}},
 		{"", {"", ""}},
 	};
 
