@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2013-2023, The PurpleI2P Project
+* Copyright (c) 2013-2024, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -29,24 +29,12 @@
 #include "CPU.h"
 
 // recognize openssl version and features
-#if (defined(LIBRESSL_VERSION_NUMBER) && (LIBRESSL_VERSION_NUMBER >= 0x3050200fL)) // LibreSSL 3.5.2 and above
-#	define LEGACY_OPENSSL 0
-#elif ((OPENSSL_VERSION_NUMBER < 0x010100000) || defined(LIBRESSL_VERSION_NUMBER)) // 1.0.2 and below or LibreSSL
-#	define LEGACY_OPENSSL 1
-#	define X509_getm_notBefore X509_get_notBefore
-#	define X509_getm_notAfter X509_get_notAfter
-#else
-#	define LEGACY_OPENSSL 0
-#	if (OPENSSL_VERSION_NUMBER >= 0x010101000) // 1.1.1
-#		define OPENSSL_HKDF 1
-#		define OPENSSL_EDDSA 1
-#		define OPENSSL_X25519 1
-#		if (OPENSSL_VERSION_NUMBER != 0x030000000) // 3.0.0, regression in SipHash
-#			define OPENSSL_SIPHASH 1
-#		endif
-#	endif
-#	if !defined OPENSSL_NO_CHACHA && !defined OPENSSL_NO_POLY1305 // some builds might not include them
-#		define OPENSSL_AEAD_CHACHA20_POLY1305 1
+#if (OPENSSL_VERSION_NUMBER >= 0x010101000) // 1.1.1
+#	define OPENSSL_HKDF 1
+#	define OPENSSL_EDDSA 1
+#	define OPENSSL_X25519 1
+#	if (OPENSSL_VERSION_NUMBER != 0x030000000) // 3.0.0, regression in SipHash
+#		define OPENSSL_SIPHASH 1
 #	endif
 #endif
 
@@ -312,79 +300,4 @@ namespace crypto
 }
 }
 
-// take care about openssl below 1.1.0
-#if LEGACY_OPENSSL
-// define getters and setters introduced in 1.1.0
-inline int DSA_set0_pqg(DSA *d, BIGNUM *p, BIGNUM *q, BIGNUM *g)
-	{
-		if (d->p) BN_free (d->p);
-		if (d->q) BN_free (d->q);
-		if (d->g) BN_free (d->g);
-		d->p = p; d->q = q; d->g = g; return 1;
-	}
-inline int DSA_set0_key(DSA *d, BIGNUM *pub_key, BIGNUM *priv_key)
-	{
-		if (d->pub_key) BN_free (d->pub_key);
-		if (d->priv_key) BN_free (d->priv_key);
-		d->pub_key = pub_key; d->priv_key = priv_key; return 1;
-	}
-inline void DSA_get0_key(const DSA *d, const BIGNUM **pub_key, const BIGNUM **priv_key)
-	{ *pub_key = d->pub_key; *priv_key = d->priv_key; }
-inline int DSA_SIG_set0(DSA_SIG *sig, BIGNUM *r, BIGNUM *s)
-	{
-		if (sig->r) BN_free (sig->r);
-		if (sig->s) BN_free (sig->s);
-		sig->r = r; sig->s = s; return 1;
-	}
-inline void DSA_SIG_get0(const DSA_SIG *sig, const BIGNUM **pr, const BIGNUM **ps)
-	{ *pr = sig->r; *ps = sig->s; }
-
-inline int ECDSA_SIG_set0(ECDSA_SIG *sig, BIGNUM *r, BIGNUM *s)
-	{
-		if (sig->r) BN_free (sig->r);
-		if (sig->s) BN_free (sig->s);
-		sig->r = r; sig->s = s; return 1;
-	}
-inline void ECDSA_SIG_get0(const ECDSA_SIG *sig, const BIGNUM **pr, const BIGNUM **ps)
-	{ *pr = sig->r; *ps = sig->s; }
-
-inline int RSA_set0_key(RSA *r, BIGNUM *n, BIGNUM *e, BIGNUM *d)
-	{
-		if (r->n) BN_free (r->n);
-		if (r->e) BN_free (r->e);
-		if (r->d) BN_free (r->d);
-		r->n = n; r->e = e; r->d = d; return 1;
-	}
-inline void RSA_get0_key(const RSA *r, const BIGNUM **n, const BIGNUM **e, const BIGNUM **d)
-	{ *n = r->n; *e = r->e; *d = r->d; }
-
-inline int DH_set0_pqg(DH *dh, BIGNUM *p, BIGNUM *q, BIGNUM *g)
-	{
-		if (dh->p) BN_free (dh->p);
-		if (dh->q) BN_free (dh->q);
-		if (dh->g) BN_free (dh->g);
-		dh->p = p; dh->q = q; dh->g = g; return 1;
-	}
-inline int DH_set0_key(DH *dh, BIGNUM *pub_key, BIGNUM *priv_key)
-	{
-		if (dh->pub_key) BN_free (dh->pub_key);
-		if (dh->priv_key) BN_free (dh->priv_key);
-		dh->pub_key = pub_key; dh->priv_key = priv_key; return 1;
-	}
-inline void DH_get0_key(const DH *dh, const BIGNUM **pub_key, const BIGNUM **priv_key)
-	{ *pub_key = dh->pub_key; *priv_key = dh->priv_key; }
-
-inline RSA *EVP_PKEY_get0_RSA(EVP_PKEY *pkey)
-	{ return pkey->pkey.rsa; }
-
-inline EVP_MD_CTX *EVP_MD_CTX_new ()
-	{ return EVP_MD_CTX_create(); }
-inline void EVP_MD_CTX_free (EVP_MD_CTX *ctx)
-	{ EVP_MD_CTX_destroy (ctx); }
-
-// ssl
-#define TLS_method TLSv1_method
-
-#endif
-
 #endif
