@@ -118,9 +118,9 @@ namespace data
 	{
 		i2p::util::SetThreadName("NetDB");
 
-		uint64_t lastManage = 0, lastExploratory = 0, lastManageRequest = 0;
+		uint64_t lastManage = 0;
 		uint64_t lastProfilesCleanup = i2p::util::GetMonotonicMilliseconds (), lastObsoleteProfilesCleanup = lastProfilesCleanup;
-		int16_t profilesCleanupVariance = 0, obsoleteProfilesCleanVariance = 0, exploratoryIntervalVariance = 0;
+		int16_t profilesCleanupVariance = 0, obsoleteProfilesCleanVariance = 0;
 
 		while (m_IsRunning)
 		{
@@ -138,9 +138,6 @@ namespace data
 							case eI2NPDatabaseStore:
 								HandleDatabaseStoreMsg (msg);
 							break;
-							case eI2NPDatabaseSearchReply:
-								HandleDatabaseSearchReplyMsg (msg);
-							break;
 							case eI2NPDatabaseLookup:
 								HandleDatabaseLookupMsg (msg);
 							break;
@@ -162,15 +159,6 @@ namespace data
 					continue; // don't manage netdb when offline or transports are not running
 
 				uint64_t mts = i2p::util::GetMonotonicMilliseconds ();
-				if (mts >= lastManageRequest + MANAGE_REQUESTS_INTERVAL*1000)
-				{
-					if (lastManageRequest || i2p::tunnel::tunnels.GetExploratoryPool ()) // expolratory pool is ready?
-					{	
-						if (m_Requests) m_Requests->ManageRequests ();
-						lastManageRequest = mts;
-					}	
-				}
-
 				if (mts >= lastManage + 60000) // manage routers and leasesets every minute
 				{
 					if (lastManage)
@@ -216,27 +204,6 @@ namespace data
 					lastObsoleteProfilesCleanup = mts;
 					obsoleteProfilesCleanVariance = rand () % i2p::data::PEER_PROFILE_OBSOLETE_PROFILES_CLEAN_VARIANCE;
 				}	
-			
-				if (mts >= lastExploratory + NETDB_EXPLORATORY_INTERVAL*1000LL) // check exploratory every 55 seconds
-				{
-					auto numRouters = m_RouterInfos.size ();
-					if (!numRouters)
-						throw std::runtime_error("No known routers, reseed seems to be totally failed");
-					else // we have peers now
-						m_FloodfillBootstrap = nullptr;
-					if (numRouters < 2500 || mts >= lastExploratory + (NETDB_EXPLORATORY_INTERVAL + exploratoryIntervalVariance)*1000LL) 
-					{
-						if(!i2p::context.IsHidden ())
-						{	
-							numRouters = 800/numRouters;
-							if (numRouters < 1) numRouters = 1;
-							if (numRouters > 9) numRouters = 9;
-							Explore (numRouters);
-						}	
-						lastExploratory = mts;
-						exploratoryIntervalVariance = rand () % NETDB_EXPLORATORY_INTERVAL_VARIANCE;
-					}
-				}
 			}
 			catch (std::exception& ex)
 			{
@@ -511,28 +478,6 @@ namespace data
 			m_Reseeder->LoadCertificates (); // we need certificates for SU3 verification
 		}
 
-		// try reseeding from floodfill first if specified
-		std::string riPath; i2p::config::GetOption("reseed.floodfill", riPath);
-		if (!riPath.empty())
-		{
-			auto ri = std::make_shared<RouterInfo>(riPath);
-			if (ri->IsFloodfill())
-			{
-				const uint8_t * riData = ri->GetBuffer();
-				int riLen = ri->GetBufferLen();
-				if (!i2p::data::netdb.AddRouterInfo(riData, riLen))
-				{
-					// bad router info
-					LogPrint(eLogError, "NetDb: Bad router info");
-					return;
-				}
-				m_FloodfillBootstrap = ri;
-				//ReseedFromFloodfill(*ri);
-				// don't try reseed servers if trying to bootstrap from floodfill
-				return;
-			}
-		}
-
 		m_Reseeder->Bootstrap ();
 	}
 
@@ -819,68 +764,10 @@ namespace data
 	void NetDb::RequestDestination (const IdentHash& destination, RequestedDestination::RequestComplete requestComplete, bool direct)
 	{
 		if (direct && i2p::transport::transports.RoutesRestricted ()) direct = false; // always use tunnels for restricted routes
-		auto dest = m_Requests->CreateRequest (destination, false, direct, requestComplete); // non-exploratory
-		if (!dest)
-		{
-			LogPrint (eLogWarning, "NetDb: Destination ", destination.ToBase64(), " is requested already or cached");
-			return;
-		}
-
-		auto floodfill = GetClosestFloodfill (destination, dest->GetExcludedPeers ());
-		if (floodfill)
-		{
-			if (direct && !floodfill->IsReachableFrom (i2p::context.GetRouterInfo ()) &&
-				!i2p::transport::transports.IsConnected (floodfill->GetIdentHash ()))
-				direct = false; // floodfill can't be reached directly
-			if (direct)
-			{
-				if (CheckLogLevel (eLogDebug))
-					LogPrint (eLogDebug, "NetDb: Request ", dest->GetDestination ().ToBase64 (), " at ", floodfill->GetIdentHash ().ToBase64 (), " directly");
-				auto msg = dest->CreateRequestMessage (floodfill->GetIdentHash ());
-				msg->onDrop = [this, dest]() { if (dest->IsActive ()) this->m_Requests->SendNextRequest (dest); }; 
-				transports.SendMessage (floodfill->GetIdentHash (), msg);
-			}	
-			else
-			{
-				auto pool = i2p::tunnel::tunnels.GetExploratoryPool ();
-				auto outbound = pool ? pool->GetNextOutboundTunnel (nullptr, floodfill->GetCompatibleTransports (false)) : nullptr;
-				auto inbound = pool ? pool->GetNextInboundTunnel (nullptr, floodfill->GetCompatibleTransports (true)) : nullptr;
-				if (outbound &&	inbound)
-				{
-					if (CheckLogLevel (eLogDebug))
-						LogPrint (eLogDebug, "NetDb: Request ", dest->GetDestination ().ToBase64 (), " at ", floodfill->GetIdentHash ().ToBase64 (), " through tunnels");
-					auto msg = dest->CreateRequestMessage (floodfill, inbound);
-					msg->onDrop = [this, dest]() { if (dest->IsActive ()) this->m_Requests->SendNextRequest (dest); }; 
-					outbound->SendTunnelDataMsgTo (floodfill->GetIdentHash (), 0,
-						i2p::garlic::WrapECIESX25519MessageForRouter (msg, floodfill->GetIdentity ()->GetEncryptionPublicKey ()));
-				}
-				else
-				{
-					LogPrint (eLogError, "NetDb: ", destination.ToBase64(), " destination requested, but no tunnels found");
-					m_Requests->RequestComplete (destination, nullptr);
-				}
-			}
-		}
+		if (m_Requests)
+			m_Requests->PostRequestDestination (destination, requestComplete, direct);
 		else
-		{
-			LogPrint (eLogError, "NetDb: ", destination.ToBase64(), " destination requested, but no floodfills found");
-			m_Requests->RequestComplete (destination, nullptr);
-		}
-	}
-
-	void NetDb::RequestDestinationFrom (const IdentHash& destination, const IdentHash & from, bool exploratory, RequestedDestination::RequestComplete requestComplete)
-	{
-
-		auto dest = m_Requests->CreateRequest (destination, exploratory, true, requestComplete); // non-exploratory
-		if (!dest)
-		{
-			LogPrint (eLogWarning, "NetDb: Destination ", destination.ToBase64(), " is requested already");
-			return;
-		}
-		if (CheckLogLevel (eLogDebug))
-			LogPrint(eLogDebug, "NetDb: Destination ", destination.ToBase64(), " being requested directly from ", from.ToBase64());
-		// direct
-		transports.SendMessage (from, dest->CreateRequestMessage (nullptr, nullptr));
+			LogPrint (eLogError, "NetDb: Requests is null");
 	}
 
 	void NetDb::HandleNTCP2RouterInfoMsg (std::shared_ptr<const I2NPMessage> m)
@@ -1025,69 +912,16 @@ namespace data
 			{
 				memcpy (payload + DATABASE_STORE_HEADER_SIZE, buf + payloadOffset, msgLen);
 				floodMsg->FillI2NPMessageHeader (eI2NPDatabaseStore);
-				Flood (ident, floodMsg);
+				int minutesBeforeMidnight = 24*60 - i2p::util::GetMinutesSinceEpoch () % (24*60);
+				bool andNextDay = storeType ? minutesBeforeMidnight < NETDB_NEXT_DAY_LEASESET_THRESHOLD:
+					minutesBeforeMidnight < NETDB_NEXT_DAY_ROUTER_INFO_THRESHOLD;
+				Flood (ident, floodMsg, andNextDay);
 			}
 			else
 				LogPrint (eLogError, "NetDb: Database store message is too long ", floodMsg->len);
 		}
 	}
-
-	void NetDb::HandleDatabaseSearchReplyMsg (std::shared_ptr<const I2NPMessage> msg)
-	{
-		const uint8_t * buf = msg->GetPayload ();
-		char key[48];
-		int l = i2p::data::ByteStreamToBase64 (buf, 32, key, 48);
-		key[l] = 0;
-		size_t num = buf[32]; // num
-		LogPrint (eLogDebug, "NetDb: DatabaseSearchReply for ", key, " num=", num);
-		IdentHash ident (buf);
-		auto dest = m_Requests->FindRequest (ident);
-		if (dest && dest->IsActive ())
-		{
-			if (!dest->IsExploratory () && (num > 0 || dest->GetNumExcludedPeers () < 3)) // before 3-rd attempt might be just bad luck
-				// try to send next requests
-				m_Requests->SendNextRequest (dest);
-			else
-				// no more requests for destination possible. delete it
-				m_Requests->RequestComplete (ident, nullptr);
-		}
-		else if (!m_FloodfillBootstrap)
-		{	
-			LogPrint (eLogInfo, "NetDb: Unsolicited or late database search reply for ", key);
-			return;
-		}	
-
-		// try responses
-		if (num > NETDB_MAX_NUM_SEARCH_REPLY_PEER_HASHES)
-		{
-			LogPrint (eLogWarning, "NetDb: Too many peer hashes ", num, " in database search reply, Reduced to ", NETDB_MAX_NUM_SEARCH_REPLY_PEER_HASHES);
-			num = NETDB_MAX_NUM_SEARCH_REPLY_PEER_HASHES;
-		}	
-		for (size_t i = 0; i < num; i++)
-		{
-			const uint8_t * router = buf + 33 + i*32;
-			char peerHash[48];
-			int l1 = i2p::data::ByteStreamToBase64 (router, 32, peerHash, 48);
-			peerHash[l1] = 0;
-			LogPrint (eLogDebug, "NetDb: ", i, ": ", peerHash);
-
-			auto r = FindRouter (router);
-			if (!r || i2p::util::GetMillisecondsSinceEpoch () > r->GetTimestamp () + 3600*1000LL)
-			{
-				// router with ident not found or too old (1 hour)
-				LogPrint (eLogDebug, "NetDb: Found new/outdated router. Requesting RouterInfo...");
-				if(m_FloodfillBootstrap)
-					RequestDestinationFrom(router, m_FloodfillBootstrap->GetIdentHash(), true);
-				else if (!IsRouterBanned (router))
-					RequestDestination (router);
-				else
-					LogPrint (eLogDebug, "NetDb: Router ", peerHash, " is banned. Skipped");
-			}
-			else
-				LogPrint (eLogDebug, "NetDb: [:|||:]");
-		}
-	}
-
+	
 	void NetDb::HandleDatabaseLookupMsg (std::shared_ptr<const I2NPMessage> msg)
 	{
 		const uint8_t * buf = msg->GetPayload ();
@@ -1131,7 +965,7 @@ namespace data
 				return;
 			}	
 			LogPrint (eLogInfo, "NetDb: Exploratory close to ", key, " ", numExcluded, " excluded");
-			std::set<IdentHash> excludedRouters;
+			std::unordered_set<IdentHash> excludedRouters;
 			const uint8_t * excluded_ident = excluded;
 			for (int i = 0; i < numExcluded; i++)
 			{
@@ -1183,7 +1017,7 @@ namespace data
 
 			if (!replyMsg)
 			{
-				std::set<IdentHash> excludedRouters;
+				std::unordered_set<IdentHash> excludedRouters;
 				const uint8_t * exclude_ident = excluded;
 				for (int i = 0; i < numExcluded; i++)
 				{
@@ -1250,74 +1084,43 @@ namespace data
 		}
 	}
 
-	void NetDb::Explore (int numDestinations)
+	void NetDb::Flood (const IdentHash& ident, std::shared_ptr<I2NPMessage> floodMsg, bool andNextDay)
 	{
-		// new requests
-		auto exploratoryPool = i2p::tunnel::tunnels.GetExploratoryPool ();
-		auto outbound = exploratoryPool ? exploratoryPool->GetNextOutboundTunnel () : nullptr;
-		auto inbound = exploratoryPool ? exploratoryPool->GetNextInboundTunnel () : nullptr;
-		bool throughTunnels = outbound && inbound;
-
-		uint8_t randomHash[32];
-		std::vector<i2p::tunnel::TunnelMessageBlock> msgs;
-		LogPrint (eLogInfo, "NetDb: Exploring new ", numDestinations, " routers ...");
-		for (int i = 0; i < numDestinations; i++)
-		{
-			RAND_bytes (randomHash, 32);
-			auto dest = m_Requests->CreateRequest (randomHash, true, !throughTunnels); // exploratory
-			if (!dest)
-			{
-				LogPrint (eLogWarning, "NetDb: Exploratory destination is requested already");
-				return;
-			}
-			auto floodfill = GetClosestFloodfill (randomHash, dest->GetExcludedPeers ());
-			if (floodfill)
-			{
-				if (i2p::transport::transports.IsConnected (floodfill->GetIdentHash ()))
-					throughTunnels = false;
-				if (throughTunnels)
-				{
-					msgs.push_back (i2p::tunnel::TunnelMessageBlock
-						{
-							i2p::tunnel::eDeliveryTypeRouter,
-							floodfill->GetIdentHash (), 0,
-							CreateDatabaseStoreMsg () // tell floodfill about us
-						});
-					msgs.push_back (i2p::tunnel::TunnelMessageBlock
-						{
-							i2p::tunnel::eDeliveryTypeRouter,
-							floodfill->GetIdentHash (), 0,
-							dest->CreateRequestMessage (floodfill, inbound) // explore
-						});
-				}
-				else
-					i2p::transport::transports.SendMessage (floodfill->GetIdentHash (), dest->CreateRequestMessage (floodfill->GetIdentHash ()));
-			}
-			else
-				m_Requests->RequestComplete (randomHash, nullptr);
-		}
-		if (throughTunnels && msgs.size () > 0)
-			outbound->SendTunnelDataMsgs (msgs);
-	}
-
-	void NetDb::Flood (const IdentHash& ident, std::shared_ptr<I2NPMessage> floodMsg)
-	{
-		std::set<IdentHash> excluded;
+		std::unordered_set<IdentHash> excluded;
 		excluded.insert (i2p::context.GetIdentHash ()); // don't flood to itself
 		excluded.insert (ident); // don't flood back
 		for (int i = 0; i < 3; i++)
 		{
-			auto floodfill = GetClosestFloodfill (ident, excluded);
+			auto floodfill = GetClosestFloodfill (ident, excluded, false); // current day
 			if (floodfill)
 			{
-				auto h = floodfill->GetIdentHash();
-				LogPrint(eLogDebug, "NetDb: Flood lease set for ", ident.ToBase32(), " to ", h.ToBase64());
+				const auto& h = floodfill->GetIdentHash();
 				transports.SendMessage (h, CopyI2NPMessage(floodMsg));
 				excluded.insert (h);
 			}
 			else
-				break;
+				return; // no more floodfills
 		}
+		if (andNextDay)
+		{
+			// flood to two more closest flodfills for next day 
+			std::unordered_set<IdentHash> excluded1;
+			excluded1.insert (i2p::context.GetIdentHash ()); // don't flood to itself
+			excluded1.insert (ident); // don't flood back
+			for (int i = 0; i < 2; i++)
+			{
+				auto floodfill = GetClosestFloodfill (ident, excluded1, true); // next day
+				if (floodfill)
+				{
+					const auto& h = floodfill->GetIdentHash();
+					if (!excluded.count (h)) // we didn't send for current day, otherwise skip
+						transports.SendMessage (h, CopyI2NPMessage(floodMsg));
+					excluded1.insert (h);
+				}
+				else
+					return;
+			}	
+		}	
 	}
 
 	std::shared_ptr<const RouterInfo> NetDb::GetRandomRouter () const
@@ -1343,7 +1146,7 @@ namespace data
 			});
 	}
 
-	std::shared_ptr<const RouterInfo> NetDb::GetRandomSSU2PeerTestRouter (bool v4, const std::set<IdentHash>& excluded) const
+	std::shared_ptr<const RouterInfo> NetDb::GetRandomSSU2PeerTestRouter (bool v4, const std::unordered_set<IdentHash>& excluded) const
 	{
 		return GetRandomRouter (
 			[v4, &excluded](std::shared_ptr<const RouterInfo> router)->bool
@@ -1353,7 +1156,7 @@ namespace data
 			});
 	}
 
-	std::shared_ptr<const RouterInfo> NetDb::GetRandomSSU2Introducer (bool v4, const std::set<IdentHash>& excluded) const
+	std::shared_ptr<const RouterInfo> NetDb::GetRandomSSU2Introducer (bool v4, const std::unordered_set<IdentHash>& excluded) const
 	{
 		return GetRandomRouter (
 			[v4, &excluded](std::shared_ptr<const RouterInfo> router)->bool
@@ -1448,10 +1251,16 @@ namespace data
 		if (msg) m_Queue.Put (msg);
 	}
 
-	std::shared_ptr<const RouterInfo> NetDb::GetClosestFloodfill (const IdentHash& destination,
-		const std::set<IdentHash>& excluded) const
+	void NetDb::PostDatabaseSearchReplyMsg (std::shared_ptr<const I2NPMessage> msg)
 	{
-		IdentHash destKey = CreateRoutingKey (destination);
+		if (msg && m_Requests)
+			m_Requests->PostDatabaseSearchReplyMsg (msg);
+	}	
+	
+	std::shared_ptr<const RouterInfo> NetDb::GetClosestFloodfill (const IdentHash& destination,
+		const std::unordered_set<IdentHash>& excluded, bool nextDay) const
+	{
+		IdentHash destKey = CreateRoutingKey (destination, nextDay);
 		std::lock_guard<std::mutex> l(m_FloodfillsMutex);
 		return m_Floodfills.FindClosest (destKey, [&excluded](const std::shared_ptr<RouterInfo>& r)->bool
 			{
@@ -1461,7 +1270,7 @@ namespace data
 	}
 
 	std::vector<IdentHash> NetDb::GetClosestFloodfills (const IdentHash& destination, size_t num,
-		std::set<IdentHash>& excluded, bool closeThanUsOnly) const
+		std::unordered_set<IdentHash>& excluded, bool closeThanUsOnly) const
 	{
 		std::vector<IdentHash> res;
 		IdentHash destKey = CreateRoutingKey (destination);
@@ -1496,7 +1305,7 @@ namespace data
 	}
 
 	std::vector<IdentHash> NetDb::GetExploratoryNonFloodfill (const IdentHash& destination, 
-		size_t num, const std::set<IdentHash>& excluded)
+		size_t num, const std::unordered_set<IdentHash>& excluded)
 	{
 		std::vector<IdentHash> ret;
 		if (!num || m_RouterInfos.empty ()) return ret; // empty list
