@@ -1014,17 +1014,24 @@ namespace data
 
 	bool RouterInfo::IsEligibleFloodfill () const
 	{
-		// floodfill must have published ipv4, >= 0.9.38 and not DSA
-		return m_Version >= NETDB_MIN_FLOODFILL_VERSION && IsPublished (true) && 
+		// floodfill must have published ipv4 or reachable ipv4 and published ipv6
+		// >= 0.9.59 and not DSA
+		return m_Version >= NETDB_MIN_FLOODFILL_VERSION && (IsPublished (true) ||
+			(IsReachableBy (eNTCP2V4 | eSSU2V4) && IsPublished (false))) &&
 			GetIdentity ()->GetSigningKeyType () != SIGNING_KEY_TYPE_DSA_SHA1;
 	}
 
 	bool RouterInfo::IsPublished (bool v4) const
 	{
 		if (m_Caps & (eUnreachable | eHidden)) return false; // if router sets U or H we assume that all addresses are not published
-		return m_PublishedTransports & (v4 ? (eNTCP2V4 | eSSU2V4) : (eNTCP2V6 | eSSU2V6));
+		return IsPublishedOn (v4 ? (eNTCP2V4 | eSSU2V4) : (eNTCP2V6 | eSSU2V6));
 	}	
 
+	bool RouterInfo::IsPublishedOn (CompatibleTransports transports) const
+	{
+		return m_PublishedTransports & transports;
+	}
+	
 	bool RouterInfo::IsNAT2NATOnly (const RouterInfo& other) const
 	{
 		return !(m_PublishedTransports & other.m_SupportedTransports) &&
@@ -1120,6 +1127,12 @@ namespace data
 		m_Buffer->SetBufferLen (len);
 	}
 
+	std::shared_ptr<RouterInfo::Buffer> RouterInfo::CopyBuffer () const
+	{
+		if (!m_Buffer) return nullptr;
+		return netdb.NewRouterInfoBuffer (*m_Buffer);
+	}	
+		
 	std::shared_ptr<RouterInfo::Buffer> RouterInfo::NewBuffer () const
 	{
 		return netdb.NewRouterInfoBuffer ();
